package com.android.gl2jni;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLContext;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.PixelFormat;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.GLUtils;
import android.util.Log;

class GL2JNIView extends GLSurfaceView {

	public GL2JNIView(Context context) {
		super(context);
		init(false, 0, 0);
	}

	public GL2JNIView(Context context, boolean translucent, int depth,
			int stencil) {
		super(context);
		init(translucent, depth, stencil);
	}

	private void init(boolean translucent, int depth, int stencil) {
		if (translucent) {
			this.getHolder().setFormat(PixelFormat.TRANSLUCENT);
		}
		setEGLContextFactory(new ContextFactory());
		setEGLContextClientVersion(2);
		setRenderer(new Renderer());
	}

	private static class ContextFactory implements
			GLSurfaceView.EGLContextFactory {

		private static int EGL_CONTEXT_CLIENT_VERSION = 0x3098;

		public EGLContext createContext(EGL10 egl, EGLDisplay display,
				EGLConfig eglConfig) {
			int[] attrib_list = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL10.EGL_NONE };
			EGLContext context = egl.eglCreateContext(display, eglConfig,
					EGL10.EGL_NO_CONTEXT, attrib_list);
			return context;
		}

		public void destroyContext(EGL10 egl, EGLDisplay display,
				EGLContext context) {
			egl.eglDestroyContext(display, context);
		}
	}

	private class Renderer implements GLSurfaceView.Renderer {
		public void onDrawFrame(GL10 gl) {
			GL2JNILib.step();
		}

		public void onSurfaceChanged(GL10 gl, int width, int height) {
			GL2JNILib.init(width, height);
		}

		private Context mContext;
		int textureId;
		private int[] TextureString = new int[1];

		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			mContext = GL2JNIView.this.getContext();
			// Bitmap bitmap = getBitmap(mContext,R.drawable.bac);
			Bitmap bitmap = getBitmap(mContext, R.drawable.bac);
			if (bitmap != null) {
				Log.e("step", "bing the texture succeed!");
				gl.glEnable(GLES20.GL_TEXTURE_2D);
				gl.glGenTextures(1, TextureString, 0);
				textureId = TextureString[0];
				Log.e("textureId", String.valueOf(textureId));
				gl.glBindTexture(GLES20.GL_TEXTURE_2D, textureId);
				gl.glTexParameterf(GLES20.GL_TEXTURE_2D,
						GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_NEAREST);
				gl.glTexParameterf(GLES20.GL_TEXTURE_2D,
						GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_LINEAR);
				gl.glTexParameterf(GLES20.GL_TEXTURE_2D,
						GLES20.GL_TEXTURE_WRAP_S, GLES20.GL_CLAMP_TO_EDGE);
				gl.glTexParameterf(GLES20.GL_TEXTURE_2D,
						GLES20.GL_TEXTURE_WRAP_T, GLES20.GL_CLAMP_TO_EDGE);
				GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, bitmap, 0);
				GL2JNILib.setTextures(TextureString);
				// GL2JNILib.setTextures(textureId);
				bitmap.recycle();
			}

		}

		private Bitmap getBitmap(Context context, int resId) {
			// getBitmap by decodeResources()
			BitmapFactory.Options options = new BitmapFactory.Options();
			options.inScaled = false;
			return BitmapFactory.decodeResource(context.getResources(), resId,
					options);
		}
	}
}
